# -*- coding: utf-8 -*-
##############################################################################
#
#    Roberto Lizana Ramirez
#    Copyright (C) 2011 trey (<http://www.trey.es>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import osv, fields
from tools import config
from tools.translate import _
import netsvc
import pooler

import atom
import gdata.contacts
import gdata.contacts.service


class google_contact_account(osv.osv):
    _name = "google.contact_account"
    _description = "Google Contact Account"
    _columns = {
        'name': fields.char('User', size=255, required=True,select=1),
        'password': fields.char('Password', size=255, required=True,),
        'active': fields.boolean('Active'),
    }
    _sql_constraints = []
    _order = 'name asc'
    _defaults = {
        'active': lambda *a: True,
    }

    def check_login(self, cr, uid, ids, context=None):
        account = self.browse(cr,uid,ids[0])
        test = gdata.contacts.service.ContactsService()
        test.email = account.name
        test.password = account.password
        try:
            test.ProgrammaticLogin()
            raise osv.except_osv( _("Connection OK"), _("Your login information is correct.") )
        except gdata.service.CaptchaRequired:
            raise osv.except_osv( _("Error"), _("Required Captcha.") )
        except gdata.service.BadAuthentication:
            raise osv.except_osv( _("Error"), _("Bad Authentication.") )
        except gdata.service.Error:
            raise osv.except_osv( _("Error"), _("Login Error.") )

        
google_contact_account()

class google_contact_buffer(osv.osv):
    _name = "google.contact_buffer"
    _description = "Sync Buffer"
    _columns = {
        'action': fields.selection([('unlink','Delete'), ('update','Update'), ('create','Create')], 'Action'),
        'contact_id': fields.many2one('res.partner.contact','Contacto'),
        'google_account_id': fields.many2one('google.contact_account','Cuenta de google'),
    }
    _sql_constraints = []
    _defaults = {
    }
google_contact_buffer()

### INHERITED ###
class res_partner_contact(osv.osv):
    _inherit = 'res.partner.contact'
    _columns = {
        'google_account_id': fields.many2one('google.contact_account','Google Account', size=255),
        'google_id': fields.char('Google ID',size=255),
        'google_updated': fields.char('Google updated',size=255),
    }

    def create(self, cr, uid, vals, context=None):
        id = super(res_partner_contact, self).create(cr, uid, vals, context)
        contact = self.browse(cr,uid,id)

        if contact.google_account_id and not contact.google_id :
            pooler.get_pool(cr.dbname).get('google.contact_buffer').create(cr,uid,{
                'action': 'create',
                'contact_id': id,
                'google_account_id': contact.google_account_id.id
            })
            cr.commit()

    def write(self, cr, uid, ids, vals, context=None):
        super(res_partner_contact, self).write(cr, uid, ids, vals, context=context)

        if not isinstance(ids, (list, tuple)) :
            ids = [ids]
        
        for contact in self.browse(cr,uid,ids) :
            if contact.google_account_id :
                pooler.get_pool(cr.dbname).get('google.contact_buffer').create(cr,uid,{
                    'action': 'update',
                    'contact_id': contact.id,
                    'google_account_id': contact.google_account_id.id
                })
                cr.commit()

    def unlink(self, cr, uid, ids, context=None):
        if not isinstance(ids, (list, tuple)) :
            ids = [ids]

        for contact in self.browse(cr,uid,ids) :
            if contact.google_account_id :
                pooler.get_pool(cr.dbname).get('google.contact_buffer').create(cr,uid,{
                    'action': 'unlink',
                    'contact_id': contact.id,
                    'google_account_id': contact.google_account_id.id
                })
                cr.commit()

        super(res_partner_contact, self).unlink(cr, uid, ids, context=context)

res_partner_contact()
