# -*- encoding: utf-8 -*-
#################################################################################
#    Autor: Roberto Lizana (robertolizana@trey.es)
#    Copyright (C) 2010 Trey (<http://www.trey.es>). All Rights Reserved
#       
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#################################################################################
import wizard
import pooler
from osv import fields,osv
from tools import config
from tools.translate import _
import netsvc
import time
import datetime

import atom
import gdata.contacts
import gdata.contacts.service

import dateutil
from dateutil.tz import *
from dateutil.parser import *

def create_contact_dict(gc_client,account_id,feed_entry) :
    # Set email address
    email = ''
    for e in feed_entry.email :
        if e.primary == 'true' :
            email = e.address
    if email=='' and feed_entry.email :
        email = feed_entry.email[0].address

    # Set phone
    phone = ''
    for e in feed_entry.phone_number :
        if e.primary == 'true' :
            phone = e.text
    if phone=='' and feed_entry.phone_number :
        phone = feed_entry.phone_number[0].text

    return {
        'google_account_id': account_id,
        'google_id': feed_entry.id.text,
        'google_updated': feed_entry.updated.text,
        'name': feed_entry.title.text and feed_entry.title.text or ' ',
        'email': email,
        'mobile': phone,
        'comment': feed_entry.content and feed_entry.content.text or '',
        'photo': gc_client.GetPhoto(feed_entry),
    }


def gc_account_synchronize(cr,uid,account_id) :
    pool = pooler.get_pool(cr.dbname)
    account = pool.get('google.contact_account').browse(cr,uid,account_id)

    # Login
    gc_client = gdata.contacts.service.ContactsService()
    gc_client.email = account.name
    gc_client.password = account.password
    gc_client.source = 'exampleCo-exampleApp-1'
    gc_client.ProgrammaticLogin()    
    netsvc.Logger().notifyChannel('trey_google_contact', netsvc.LOG_INFO, "Login Google Contact: %s" % account.name )

    # Init val
    create_contact = {}
    update_contact = {}
    unlink_contact = {}
        
    # Remove contacts if they has deleted in the Google Contacts OpenERP
    ids = pool.get('google.contact_buffer').search(cr,uid,[('google_account_id','=',account.id)])
    for id in ids :
        buffer = pool.get('google.contact_buffer').browse(cr,uid,id)
        if buffer.action == 'create' :
            create_contact[buffer.contact_id.google_id] = buffer.contact_id
        elif buffer.action == 'update':
            update_contact[buffer.contact_id.google_id] = buffer.contact_id
        elif buffer.action == 'unlink':
            unlink_contact[buffer.contact_id.google_id] = buffer.contact_id
        
        
    # Sync contacts
    feed = gc_client.GetContactsFeed()


    while feed:
        for f in feed.entry:

            partner_contact_ids = pool.get('res.partner.contact').search(cr,uid,[('google_id','=',f.id.text)])
            
            if f.id.text in unlink_contact :                
                netsvc.Logger().notifyChannel('trey_google_contact', netsvc.LOG_INFO, "Delete Google contact: %s" % ( f.title.text ))
                gc_client.DeleteContact(f.GetEditLink().href)
            elif f.id.text in update_contact :
                netsvc.Logger().notifyChannel('trey_google_contact', netsvc.LOG_INFO, "Update Google contact: %s" % ( f.title.text ))
                contact = pool.get('res.partner.contact').browse(cr,uid,partner_contact_ids[0])

                # Set email address
                email = None
                for e in f.email :
                    if e.primary == 'true' :
                        email = e
                if email == None and feed_entry.email :
                    email = f.email[0]

                # Set phone
                phone = None
                for e in f.phone_number :
                    if e.primary == 'true' :
                        phone = e
                if phone == None and f.phone_number :
                    phone = f.phone_number[0]

                f.title.text = contact.name
                f.content.text = contact.comment
                if email : 
                    email.address = contact.email
                if phone :
                    phone.text = contact.mobile

                gc_client.UpdateContact(f.GetEditLink().href, f)
                
            elif not partner_contact_ids :
                netsvc.Logger().notifyChannel('trey_google_contact', netsvc.LOG_INFO, "Create contact in OpenERP: %s" % ( f.title.text ))

                # Create contact                
                partner_contact = create_contact_dict(gc_client,account_id, f)
                pool.get('res.partner.contact').create(cr,uid,partner_contact)
                cr.commit()
            else :
                contact = pool.get('res.partner.contact').browse(cr,uid,partner_contact_ids[0])

                if contact.google_updated != f.updated.text :
                    #~ google_date = dateutil.parser.parse( f.updated.text )
                    #~ google_date = datetime.datetime(*google_date.timetuple()[:6])
                    netsvc.Logger().notifyChannel('trey_google_contact', netsvc.LOG_INFO, "Update OpenERP contact %s" % ( f.title.text ))
                    partner_contact = create_contact_dict(gc_client,account_id, f)
                    pool.get('res.partner.contact').write(cr,uid,contact.id,partner_contact)
                    cr.commit()
                
            

        next = feed.GetNextLink()
        feed = None
        if next:
            feed = gc_client.GetContactsFeed(next.href)

    return


def gc_create_contact():
    contact = gdata.contacts.ContactEntry(title=atom.Title(text=name))
    contact.content = atom.Content(text=notes)

    # Create a work email address for the contact and use as primary.
    contact.email.append(gdata.contacts.Email(address=primary_email, primary='true', rel=gdata.contacts.REL_WORK))

    # Add extended properties to add data which is used in your application.
    contact.extended_property.append(gdata.ExtendedProperty(name='favourite flower', value='daisy'))        
    sports_property = gdata.ExtendedProperty(name='sports')
    sports_property.SetXmlBlob('<dance><salsa/><ballroom_dancing/></dance>')
    contact.extended_property.append(sports_property)

    # Send the contact data to the server.
    contact_entry = gc_client.CreateContact(contact)    



class synchronize_contacts_wizard(wizard.interface):
    def _sync(self, cr, uid, data, context={}):
        google_account_obj = pooler.get_pool(cr.dbname).get('google.contact_account')
        for account_id in google_account_obj.search(cr,uid,[('active','=',True)]) :
            gc_account_synchronize(cr,uid,account_id)        

    states = {
        'init': {
            'actions': [],
            'result':{'type':'action', 'action': _sync, 'state':'end'}
        },
    }

synchronize_contacts_wizard('wizard_synchronize_contacts')
